const deepMerge = require('deepmerge')
const Color = require('color')
const customFormsPlugin = require('@tailwindcss/custom-forms')
const multiThemePlugin = require('tailwindcss-multi-theme')
const shadowOutlinePlugin = ({ addUtilities, theme, variants }) => {
  const newUtilities = {}
  Object.entries(theme('colors')).map(([name, value]) => {
    if (name === 'transparent' || name === 'current') return
    const color = value[300] ? value[300] : value
    const hsla = Color(color).alpha(0.45).hsl().string()

    newUtilities[`.shadow-outline-${name}`] = {
      'box-shadow': `0 0 0 3px ${hsla}`,
    }
  })

  addUtilities(newUtilities, variants('boxShadow'))
}

const colors = {
  transparent: 'transparent',
  white:    '#ffffff',
  black:    '#000000',
  primary: {
    '100':  '#ffff6e',
    '200':  '#feee00',
    '300':  '#c8bb00',
  },
  secondary: {
    '100':  '#494768',
    '200':  '#20203d',
    '300':  '#000018',
  },
  accent:   '#bf0811',
  neutral: {
    '100':  '#ffffff',
    '200':  '#ececed',
    '300':  '#d3d6de',
    '400':  '#adb0ba',
    '500':  '#4f525a',
    '600':  '#33363e',
    '700':  '#11151c',
  },
  support: {
    '100':  '#4bbaf8',
    '200':  '#11e096',
    '300':  '#ff5454',
  },
  base: {
    '100':  '#ffffff',
    '200':  '#f4f4f4',
    '300':  '#f4f4f2',
  },
}

const minWidth = (theme) => ({
  'max-content': 'max-content',
  ...theme('spacing'),
})

const backgroundOpacity = (theme) => ({
  '10': '0.1',
  ...theme('opacity'),
})

const boxShadow = (theme) => ({
  '10': '0px 8px 24px 0px rgba(0,0,0,0.16)',
  '20': '0px 8px 24px 0px rgba(0,0,0,0.08)',
  '30': '0px 8px 24px 0px rgba(0,0,0,0.16)',
  ...theme('shadow'),
})

const maxHeight = (theme) => ({
  '4':  '4px',
  '8':  '8px',
  '16': '16px',
  '24': '24px',
  '32': '32px',
  '48': '48px',
  '64': '64px',
  ...theme('spacing'),
})

const customForms = (theme) => ({
  default: {
    'input, textarea': {
      '&::placeholder': {
        color: theme('colors.gray.400'),
      },
    },
  },
})

const windmillConfig = {
  purge: {
    content: [
      'node_modules/@windmill/react-ui/lib/defaultTheme.js',
      'node_modules/@windmill/react-ui/dist/index.js',
    ],
    options: {
      whitelist: ['theme-dark'],
    },
  },
  theme: {
    themeVariants: ['dark'],
    colors,
    backgroundOpacity,
    maxHeight,
    minWidth,
    customForms,
    boxShadow
  },
  variants: {
    backgroundOpacity: ['responsive', 'hover', 'focus', 'dark:hover'],
    backgroundColor: [
      'responsive',
      'hover',
      'focus',
      'active',
      'odd',
      'dark',
      'dark:hover',
      'dark:focus',
      'dark:active',
      'dark:odd',
    ],
    display: ['responsive', 'dark'],
    textColor: [
      'responsive',
      'focus',
      'focus-within',
      'hover',
      'active',
      'dark',
      'dark:focus-within',
      'dark:hover',
      'dark:active',
    ],
    placeholderColor: ['responsive', 'focus', 'dark', 'dark:focus'],
    borderColor: ['responsive', 'hover', 'focus', 'dark', 'dark:focus', 'dark:hover'],
    divideColor: ['responsive', 'dark'],
    boxShadow: ['responsive', 'hover', 'focus', 'dark:focus'],
    radius: ['responsive', 'hover', 'focus', 'dark:focus'],
    margin: ['responsive', 'last'],
  },
  plugins: [customFormsPlugin, multiThemePlugin, shadowOutlinePlugin],
}

function arrayMergeFn(destinationArray, sourceArray) {
  return destinationArray.concat(sourceArray).reduce((acc, cur) => {
    if (acc.includes(cur)) return acc
    return [...acc, cur]
  }, [])
}

/**
 * Merge Windmill and Tailwind CSS configurations
 * @param {object} tailwindConfig - Tailwind config object
 * @return {object} new config object
 */
function wrapper(tailwindConfig) {
  let purge
  if (Array.isArray(tailwindConfig.purge)) {
    purge = {
      content: tailwindConfig.purge,
    }
  } else {
    purge = tailwindConfig.purge
  }
  return deepMerge({ ...tailwindConfig, purge }, windmillConfig, { arrayMerge: arrayMergeFn })
}

module.exports = wrapper
